; (c) 2008 by mr schnada <webmaster at schnada de>

;;; general and urological utilities -- see also
;;; http://www.scsh.net/cgi-bin/wiki.cgi?CodeSnippets

(define rest cdr)

(define (true? x) (not (not x)))

(define (id x) x)

(define (string-take-until s char/char-set/pred . start+end)
  (and-let* ((i (apply string-index s char/char-set/pred start+end)))
            (string-copy s (if (null? start+end) 0
                               (first start+end)) i)))

(define (table-values table)
  (let ((xs '()))
    (table-walk (lambda (key value) (set! xs (cons value xs))) table)
    xs))

(define (table-keys table)
  (let ((xs '()))
    (table-walk (lambda (key value) (set! xs (cons key xs))) table)
    xs))

(define (table-keys/2 table p?)
  (let ((xs '()))
    (table-walk (lambda (key value) (if (p? key value)
                                        (set! xs (cons key xs))))
                table)
    xs))

(define (caution format-string . args)
  (with-current-output-port (error-output-port)
    (display "CAUTION- ")
    (apply format #t format-string args)
    (newline)))


(define (separate-elements s xs)
  (if (null? xs) '()
      (cdr (fold-right (lambda (y ys) (cons* s y ys)) '() xs))))

(define encode-url (make-char-quotator '((#\: . "%3A") (#\/ . "%2F"))))

;; helper function -- call PROC with the (minimal) distinct tails of XS and YS:
;; (PROC XS' YS'), where XS = ZS o XS', YS = ZS o YS', and ZS is maximal.
;; NOTE the case-insensitive string comp (for the benefit of uri paths
;; with ci server component)!
(define (with-distinct-tails xs ys proc)
  ;; We could pass the prefix length to the proc in order to let it recognise
  ;; relative local uris with null common prefix.  (Absolute uris always
  ;; share at least the "" before the leading slash.)
  (let loop ((prefix-len 0) (ixes xs) (whys ys))
    (cond ((or (null? ixes) (null? whys))
           (proc ixes whys))
          ((string-ci= (first ixes) (first whys))
           (loop (+ prefix-len 1) (rest ixes) (rest whys)))
          (else
           (proc ixes whys)))))

;; helper function -- given uri-paths UP, DOWN of relative local uris, return
;; the uri-path hanging DOWN from the top level of UP
(define (climb up down)
  (and (pair? up)
       (fold (lambda (x xs) (cons ".." xs)) down (rest up))))

;; leading path components corresponding to //<server>
(define (server-prefix path-components)
  (if (< (length path-components) 3) '()
      (let ((xs (take path-components 3)))
        (if (and (string= "" (first xs))
                 (string= "" (second xs)))
            (map string-downcase xs)
            '()))))

;; path0, path1 : Proper-List(String) -- (components of) uri paths; non-empty
;; Return ``path1 modulo path0'' : Proper-List(String)
;; -- designating, relative to path0, the same resource as path1
;; -- assuming that the server prefix, if present, corresponds to
;;    the case-insensitive string //<server>
;; BUG: Identifies paths on the same server that differ only in the
;; capitalisation, [\\srv]\x\y and [\\srv]\X\y.
(define (relativise-path path0 path1)
  (assert (and (pair? path0) (pair? path1)))
  (cond ((equal? (server-prefix path0)
                 (server-prefix path1))
         (let ((relpath (with-distinct-tails path0 path1 climb)))
           (if relpath relpath (last-pair path1))))
        (else path1)))


(define (maybe-fix pre x sub)
  (if x (string-append pre x sub) ""))

(define (unparse-uri uri-components)
  (let ((protocol first)
        (path-components second)
        (search third)
        (fragmid fourth))
    (string-append (maybe-fix "" (protocol uri-components) ":")
                   (uri-path->uri (path-components uri-components))
                   (maybe-fix "?" (search uri-components) "")
                   (maybe-fix "#" (fragmid uri-components) ""))))


;; uri0, uri1 : String -- uri strings
;; Return ``uri1 modulo uri1 : String
;; -- designating, relative to uri0, the same resource as uri1.
(define (relativise-uri uri0 uri1)
  (receive (protocol0 path0 search0 fragmid0) (parse-uri uri0)
    (receive (protocol1 path1 search1 fragmid1) (parse-uri uri1)
      (if (equal? protocol0 protocol1)
          (let* ((relpath (relativise-path path0 path1))
                 (relprotocol (and (equal? relpath path1) protocol1)))
            (unparse-uri (list relprotocol
                               relpath
                               search1
                               fragmid1)))
          uri1))))


(define (local-uri-path? path-components)
  (or (< (length path-components) 2)
      (string<> "" (first path-components))
      (string<> "" (second path-components))))

(define (uri-path-root path-components)
  (append (if (local-uri-path? path-components) '()
              (take path-components 3))
          '("")))

(define (uri-root uri)
  (receive (protocol path _ _) (parse-uri uri)
    (unparse-uri (list protocol (uri-path-root path) #f #f))))


;; string -> string
;; Return sgml-equivalent string with numeric references instead of
;; (ascii) chars.  (Numbers according to Iso 10646 or so, no particular
;; encoding.  The first 255 code points correspond to Latin-1.  With
;; Ascii, we are going safe & easy.  Even a web browser mistakenly
;; interpreting the numbers wrt the char encoding of the document
;; cannot get it wrong -- if the encoding extends Ascii.)
;; E.g. "a@b.c" --> "&#97;&#64;&#98;&#46;&#99;"
(define (vercripple-string s)
  (let ((op (make-string-output-port))
        (len (string-length s))
        (ascii char-set:ascii))
    (do ((i 0 (+ i 1)))
        ((= i len)
         (string-output-port-output op))
      (if (char-set-contains? ascii (string-ref s i))
          (format op "&#~a;" (char->ascii (string-ref s i)))
          (display (string-ref s i) op)))))


;; approx
(define (atom? x)
  (not (pair? x)))

;; return the breadth-first non-false (pred subsexp)
(define (any-sexp pred sexp)
  (or (pred sexp)
      (and (pair? sexp)
           (or (any pred sexp)
               (any (lambda (x)
                      (any-sexp pred x))
                    sexp)))))

;; instead of image-dimension which seems to cause gc problems
;; assuming ImageMagick's "identify" utility to be found via PATH
(define (pic-dimension fname)
  (let-match (regexp-search (rx (+ any) " "
                                (submatch (+ numeric)) "x"
                                (submatch (+ numeric))
                                " " (+ any))
                            (run/string (identify ,fname)))
             (_ width height)
             (values (string->number width) (string->number height))))
