; (c) 2008, 2011 by mr schnada <webmaster at schnada de>

;;; living with sxml elements
;;; '(x (@ (y0 "z0") (y1 "z1")) "phew" (borg))

(define (elt-with-tag? sexp tag)
  (and (pair? sexp)
       (eqv? tag (car sexp))))

(define (elt-with-tag sexp tag)
  (any-sexp (lambda (x) (and (elt-with-tag? x tag) x))
            sexp))

(define (elt-tag elt)
  (if (pair? elt) (first elt) #f))

;; is X an attribute list, i.e. a (tagged) list (@ (<Symbol> <String>) ...) ?
;; approx
(define (attrilist? x)
  (eqv? (elt-tag x) '@))


(define (elt-body elt)
  (cond ((atom? elt) elt)
        ((null? (rest elt)) '())
        ((attrilist? (second elt)) (cddr elt))
        (else (cdr elt))))

(define elt-elements elt-body)
(define (elt-attributes elt)
  (rest (elt-attrilist elt)))

;; depth-first dilettante
(define (elt-ref elt tag)
  (define (rito? subelt) (eqv? tag (elt-tag subelt)))
  (cond ((not (pair? elt)) #f)
        ((rito? elt) elt)
        ((find (lambda (e) (elt-ref e tag))
               (rest elt)) => (lambda (e) (elt-ref e tag)))
        (else #f)))

(define (elt-attrilist elt)
  (or (elt-ref elt '@) '(@)))

(define (elt-attributes-ref elt tag)
  (elt-ref (elt-attrilist elt) tag))

(define (elt-elements-after elt tag)
  (and-let* ((subelt (elt-ref elt tag)))
            (rest (find-tail (lambda (e) (eq? subelt e)) elt))))

(define (normalise-element elt)
  (cons* (elt-tag elt) (elt-attrilist elt)
         (elt-body elt)))

#! test
(let ((equal? equal?))
  (format #t "~s~%" (list (equal? (elt-elements-after '(foo (x) (y) "a" (z))
                                                      'y)
                                  '("a" (z)))
                          (equal? (elt-elements-after '(foo (x) (y))
                                                      'y)
                                  '())
                          (equal? (elt-elements-after '(foo (x) "a" (z))
                                                      'y)
                                  #f)
                          (equal? (normalise-element '(foo))
                                  '(foo (@)))
                          (equal? (normalise-element '(foo (@)))
                                  '(foo (@)))
                          (equal? (normalise-element '(foo (@ (x "x"))))
                                  '(foo (@ (x "x")))))))
!#




