; (c) 2008, 2011 by mr schnada <webmaster at schnada de>

;;; compile sxml into reasonably pretty html
;;; i.e. with some linebreaks that don't affect the appearance
;;; goodie: 'include macro for fully expanded sxml code, html or text

;; lists some element types whose instances can stand a subsequent
;; #\newline without compromising the semantics (appearance) of the
;; HTML. Be <em>care</em>ful.
(define breaking-eltypes
  `(address blockquote body br button
    caption code colgroup dd div dl dt
    fieldset form
    h1 h2 h3 h4 h5 h6 head hr html
    label legend li link meta map noscript
    object ol optgroup option p pre
    samp script select style
    table tbody td textarea tfoot th thead title tr
    ul
    applet center dir menu))

(define (breaking-eltype? tag)
  (true? (memq tag breaking-eltypes)))

(define (entag tag elts)
  (let ((maybe-nl (and (breaking-eltype? tag)
                    #\newline)))
    (if (and (pair? elts)
             (attrilist? (first elts)))
        (list #\< tag (rest (first elts)) #\>
              (and (pair? (rest elts))
                   (list (rest elts) "</" tag #\>))
              maybe-nl)
        (list #\< tag #\>
              (and (pair? elts)
                   (list elts "</" tag #\>))
              maybe-nl
              ))))


(define (file-name-extension-sans-dot fname)
  (and-let* ((ext (file-name-extension fname)))
            (string= "." ext 0 1 0 1)
            (string-drop ext 1)))

(define (file-name-extensymbol fname)
  (and-let* ((ext (file-name-extension-sans-dot fname)))
            (string->symbol ext)))

;; ftype : Proper-List(String)
(define (include-action fname maybe-ftype)
  (case (string->symbol (:optional maybe-ftype
                                   (file-name-extension-sans-dot fname)))
    ((html) (lambda ()
              (run/string (cat ,fname))))
    ((txt) (lambda ()
             (string->goodHTML (run/string (cat ,fname)))))
    ((scm) (let ((inport (make-string-input-port
                          (run/string (cat ,fname)))))
             (lambda () (read inport))))
    (else (error "no include action for file " fname))))

(define (sxml->html elts . maybe-prefix)
  (let ((prefix (:optional maybe-prefix dt-xhtml1)))
    (SRV:send-reply
     (pre-post-order elts
       `((@
          ((*default* ; local override for attributes
            . ,(lambda (attr-key . value) (enattr attr-key value))))
          . ,(lambda (trigger . value) (cons '@ value)))
         (*default* . ,(lambda (tag . elts) (entag tag elts)))
         (*text* . ,(lambda (trigger x)
                      (cond ((symbol? x) (list (format #f "&~a;" x)))
                            ((number? x) (list (format #f "&#~a;" x)))
                            ((string? x) (string->goodHTML x))
                            (else x))))
         (final ((*text* . ,(lambda (trigger x) x)))
                . ,(lambda (tag . xs)  xs))
         (include ((*text* . ,(lambda (trigger x) x)))
                  . ,(lambda (_ fname . maybe-ftype)
                       ((include-action fname maybe-ftype))))
         ;; hack for sxml inclusion, in lieu of (include "some-sxml.scm")
         (includex *macro*
                   . ,(lambda (_ fname-text)
                        (with-input-from-file (second fname-text) read-cs))))))))
