; (c) 2008, 2013 by mr schnada <webmaster at schnada de>

;;; managing web page sources in a "database" (table)

(define-record-type web-database:
  (really-make-web-database dir page-table res-table)
  (page-table web-database-page-table)
  (res-table web-database-res-table)
  (dir web-database-dir))

(define (make-web-database dir-string)
  (really-make-web-database dir-string
                            (make-string-table)
                            (make-string-table)))

(define-record-type page:
  (make-page nickname code default-fragment) page?
  (nickname page-nickname)
  (code page-code)
  (default-fragment page-default-fragment))

(define (path->nickname path)
  (file-name-sans-extension (file-name-nondirectory path)))

(define (web-database-set! db label page)
  (table-set! (web-database-page-table db) label page))

(define (page-record db label)
  (table-ref (web-database-page-table db) label))

(define (page-label? db string)
  (true? (table-ref (web-database-page-table db) string)))

(define (page-code/2 db label)
  (and-let* ((page (page-record db label)))
    (page-code page)))

(define (page-default-fragment/2 db label)
  (and-let* ((page (page-record db label)))
    (page-default-fragment page)))

(define (eltofpage-ref db label tag)
  (elt-ref (page-code/2 db label) tag))


(define (page-title/2 db label)
  (and-let* ((title (eltofpage-ref db label 'title)))
            (elt-body title)))

;; ...
(define (trusty-local-reference? string)
  (member (file-name-extension string)
          '(".jpg" ".css" ".png" ".gif" ".ico")))

;; label --a page label or a page's uri
(define (page-uri db label)
  (let ((maybe-uri (eltofpage-ref db label 'loco)))
    (cond (maybe-uri (second maybe-uri))
          ((trusty-local-reference? label) label)
          (else (caution "page-uri : ~s is no page index." label)
                label))))

(define (relative-uri db base-page page . maybe-fragmid)
  (let ((uri-w/o-fragmid (relativise-uri (page-uri db base-page)
                                         (page-uri db page))))
    (if (null? maybe-fragmid)
        uri-w/o-fragmid
        (string-append uri-w/o-fragmid "#" (first maybe-fragmid)))))

; '("www.fog.net" "shag" "oomph.html")
(define (page-path db label)
  (receive (_ path _ _) (parse-uri (page-uri db label))
    (drop path 2)))

(define (page-filename db label)
  (path-list->file-name (page-path db label)))

(define (uri->file-name uri db label)
  (string-append (file-name-directory (page-filename db label))
                 (relative-uri db label uri)))

;; -> String
(define (page-protocol db label)
  (let ((maybe-uri (eltofpage-ref db label 'loco)))
    (cond (maybe-uri (string-take-until (second maybe-uri) #\:))
          (else (caution "page-protocol : ~s is no page index." label)
                label))))

(define (page-server db label)
  (first (page-path db label)))

;; testament to bad interfacing / badly needed refactoring
(define (page-server/1 page-code)
  (receive (_ path _ _) (parse-uri (second (elt-ref page-code 'loco)))
    (first (drop path 2))))

(define (page-server-description db label)
  (let-match (regexp-search (rx (submatch (+ alphanumeric))
                                "://www."
                                (submatch (+ (~ "/"))))
                            (page-uri db label))
             (_ protocol server)
             (string-append server
                            (if (equal? "https" protocol) "/SSL" ""))))

(define (web-database-set-res! db label xs)
  (table-set! (web-database-res-table db) label xs))

(define (web-database-pages db . maybe-server)
  (let ((pages (table-values (web-database-page-table db))))
    (if (null? maybe-server) pages
        (filter (lambda (page)
                  (equal? (page-server page)
                          (first maybe-server)))
                pages))))

(define (web-database-nicknames db server)
  (table-keys/2 (web-database-page-table db)
                (lambda (label page)
                  (equal? server
                          (page-server db label)))))

(define (res db label)
  (cond ((table-ref (web-database-res-table db) label) => id)
        (else (caution "res : ~s is no res label." label)
              (list label))))

(define (res-url res)
  (and (> (length res) 1)
       (second res)))

(define res-text first)


(define (quotes-id->filename id)
  (string-append "quotes/" id ".scm"))

(define (quotes db label)
  (with-cwd (web-database-dir db)
    (with-input-from-file (quotes-id->filename label)
      (lambda () (read-cs)))))

(define (quotes/name db label)
  (let ((code (quotes db label)))
    `(quol (@ (id ,label) ,@(rest (elt-attrilist code)))
           ,@(elt-body code))))


;; from the snippets wiki
(define (create-directory/parents fname . perms-override?)
    (let ((comps (split-file-name fname)))
      (let lp ((comps comps))
        (if (not (null? comps))
            (let ((parent (drop-right comps 1)))
              (lp parent)
              (if (file-not-exists? (path-list->file-name comps))
                  (apply create-directory
                         (path-list->file-name comps) perms-override?)))))))

(define (ensure-parent-dirs fname)
  (create-directory/parents (file-name-directory fname))
  fname)

;; quirky in order to accomodate legacy web pages
(define (purl db label plabel . maybe-fragmid)
  (let* ((fragmid (:optional maybe-fragmid #f))
         (eff-fragmid (if (and fragmid
                               (not (string-null? fragmid)))
                          fragmid
                          (page-default-fragment/2 db plabel)))
         (uri (relative-uri db label plabel)))
    (if eff-fragmid
        (string-append uri "#" eff-fragmid)
        uri)))


;;; handling the source dir
(define (enter-page-into-db filename db page-code->default-fragment)
  (with-input-from-file filename
    (lambda ()
      (let ((nn (path->nickname filename)))
        (format #t "Enter ~s ...~%" nn)
        (let ((page-code (read-cs)))
        (web-database-set! db nn
                           (make-page nn page-code
                                      (page-code->default-fragment
                                       page-code))))))))

(define (enter-restable-into-db restable-fname db)
  (with-input-from-file restable-fname
    (lambda ()
      (for-each (lambda (res)
                  (web-database-set-res! db (second res)
                                         (cddr res)))
                (rest (read-cs))))))

(define (enter-dir-into-db dirname db page-code->default-fragment)
  (with-cwd dirname
    (for-each (lambda (fname)
                (enter-page-into-db fname db page-code->default-fragment))
              (glob "*.scm"))
    (enter-restable-into-db "res/res.scm" db)
    db))
